/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvip.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVIP_H
#define QVIP_H

#include <iostream>

#include <qvdefines.h>
#include <QVImage>
//#include <qvcore/qvimage.h>

/// @ingroup qvip
/// @todo document this
void FilterHarrisCornerResponseImage(const QVImage<uChar> &image, QVImage<sFloat> &result, int aperture=3, int avgwindow=5, const QPoint &destROIOffset = QPoint(0,0));

/// @ingroup qvip
/// @todo document this
void FilterDoG(const QVImage<uChar> &image, QVImage<sFloat> &result);

/// @ingroup qvip
/// @deprecated see @ref FilterHessianCornerResponseImage
/// @todo document this
void SobelCornerResponseImage(const QVImage<sFloat> &image, QVImage<sFloat> &result);

/// @ingroup qvip
/// @todo document this
void FilterHessianCornerResponseImage(	const QVImage<sFloat> &image, QVImage<sFloat> &result,
					const IppiMaskSize maskSize = ippMskSize3x3,
					const QPoint &destROIOffset = QPoint(0,0));

/// @ingroup qvip
/// @todo document this
//int myFloodFill(QVImage<uChar> &image, uInt x, uInt y, uInt value, uInt minVal, uInt maxVal);

/// @ingroup qvip
/// @todo document this
void FilterNormalize(const QVImage<uChar,1> &image, QVImage<uChar,1> &equalized, const QPoint &destROIOffset = QPoint(0,0));

/// @ingroup qvip
/// @todo document this
void FilterNormalize(const QVImage<sFloat,1> &image, QVImage<sFloat,1> &equalized, const QPoint &destROIOffset = QPoint(0,0));

/// @ingroup qvip
class QVComponentTree;
void FilterPruneComponentTreeSmallRegions(QVImage<uChar> &image, QVComponentTree &componentTree, uInt area);

#include <qvmath/qvvector.h>
#include <QPoint>
/// @ingroup qvip
void FilterSeparable(const QVImage<sFloat, 1> &image, QVImage<sFloat, 1> &dest,
	const QVVector &rowFilter, const QVVector &colFilter, const QPoint &destROIOffset = QPoint(0,0));

/// @brief Generates a binary image of local grey-scale maximal value pixels.
/// @ingroup qvip
///
/// This function receives a QVImage, and generates a binary image where each pixel is set to IPP_MAX_8U if the pixel in the
/// original image is strict maximal in value regarding to pixels in a vicinity window of colMaskSize width, and rowMaskSize
/// height.
///
/// \todo
///	- Fix resulting image ROI, obtain maximums restricted to the ROI.
///	- Use IPP functions to obtain the result.
///
/// @param src source image.
/// @param dest binary image that will contain maximal values.
/// @param colMaskSize width of the vicinity.
/// @param rowMaskSize height of the vicinity.
void FilterLocalMax(const QVImage<sFloat> &src, QVImage<uChar> &dest, uInt colMaskSize, uInt rowMaskSize, sFloat threshold = 0);

/// @todo document this
/// @ingroup qvip
QMap<sFloat, QPointF> GetMaximalResponsePoints(const QVImage<sFloat> &cornerResponseImage, const double threshold = 1.0, const int windowSize = 2);

/// @todo document this
/// @ingroup qvip
/// @deprecated use @ref GetMaximalResponsePoints instead.
//QList<QPointF> GetMaximalResponsePoints1(const QVImage<sFloat> &cornerResponseImage, const double threshold = 1.0);

/// @todo document this
/// @ingroup qvip
/// @deprecated use @ref GetMaximalResponsePoints instead.
//QList<QPointF> GetMaximalResponsePoints3(const QVImage<sFloat> &cornerResponseImage, const double threshold = 1.0);

/// @brief Sorts pixels in an image, given their gray-scale value.
/// @ingroup qvip
///
/// This function obtains a QVector, that maps any gray-scale values (0..255) to the set of QPoints that hold that gray-scale
/// value, in a given image. It can be used for very fast pixel sorting.
///
/// The function itself is based in \htmlonly <a href="http://en.wikipedia.org/wiki/Counting_sort">Counting Sort algorithm</a> \endhtmlonly, so the cost of this function is O(number_of_columns * number_of_rows).
///
/// @param img source image to obtain sorted lists of QPoints.
/// @return A QVector, indexed by the gray-scale value (0-255), to a QVector of QPoints, containing the corresponding points to the pixels that hold that gray-scale value in the image.
QVector< QVector< QPoint > > CountingSort(const QVImage<uChar, 1> &img);
#endif

